package com.yh.saas.plugin.yiliangyiyun.entity;

import java.util.Date;
import java.io.Serializable;
import java.util.List;

import com.baomidou.mybatisplus.annotations.TableField;
import com.baomidou.mybatisplus.annotations.TableId;
import com.baomidou.mybatisplus.annotations.TableName;

import com.baomidou.mybatisplus.enums.IdType;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.yh.saas.common.support.entity.BaseModel;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

/**
 * <p>
 * 出入库任务
 * </p>
 *
 * @author Gongdc
 * @since 2021-06-22
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
@TableName("in_out_warehouse_task")
public class InOutWarehouseTask extends BaseModel<InOutWarehouseTask> {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @TableId(type = IdType.UUID)
    private String id;
    /**
     * 公司id
     */
    private String compId;
    /**
     * 合同编号
     */
    private String contractNo;
    /**
     * 出入库任务编号
     */
    private String inOutTaskNo;
    /**
     * 移库任务编号
     */
    private String moveTaskNo;
    /**
     * 仓库名称
     */
    private String warehouseName;
    /**
     * 仓位编号id
     */
    private String positionId;
    /**
     * 货源id
     */
    private String sourceId;
    /**
     * 仓位编号
     */
    private String binNumber;
    /**
     * 关联id
     */
    private String relevanceId;
    /**
     * 任务类型key（1出库2入库3移库4退库并出库）
     */
    private String taskTypeKey;
    /**
     * 任务类型
     */
    private String taskType;
    /**
     * 任务状态key（1待审核3已驳回5执行中7已完成）
     */
    private String taskStatusKey;
    /**
     * 任务状态
     */
    private String taskStatus;
    /**
     * 出入库类型key
     */
    private String inOutTypeKey;
    /**
     * 出入库类型
     */
    private String inOutType;
    /**
     * 出入库标识（1出库2入库）
     */
    private String inOutFlag;
    /**
     * 品级key
     */
    private String gradeKey;
    /**
     * 品级
     */
    private String grade;
    /**
     * 经办人key
     */
    private String agentKey;
    /**
     * 经办人
     */
    private String agent;
    /**
     * 货名key
     */
    private String goodsNameKey;
    /**
     * 货名
     */
    private String goodsName;
    /**
     * 出库量（吨）
     */
    private Float deliveryVolume;
    /**
     * 入库量（吨）
     */
    private Float storageVolume;
    /**
     * 退库量（吨）
     */
    private Float stockReturn;
    /**
     * 已完成量（吨）
     */
    private Double completedQuantity;
    /**
     * 运输损耗（吨）
     */
    private Float transportLoss;
    /**
     * 重量（吨）
     */
    private Float weight;
    /**
     * 单价（元/吨）
     */
    private Float unitPrice;
    /**
     * 预计日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date predictDate;
    /**
     * 创建（发布）时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date establishDate;
    /**
     * 容重
     */
    private String bulkDensity;
    /**
     * 水分
     */
    private String waterContent;
    /**
     * 业务描述
     */
    private String businessDescribe;
    /**
     * 驳回原因
     */
    private String rejectReason;
    /**
     * 发货库
     */
    private String sendWarehouse;
    /**
     * 收货库
     */
    private String receiveWarehouse;
    /**
     * 发布者
     */
    private String publisher;

    /**
     * 工作流
     */
    private String workflowId;

    /**
     * 审核状态
     */
    private String approveStatus;
    /**
     * 移库入库删除标识
     */
    private String moveDeleteFlag;
    /**
     * 移库入库仓库
     */
    private String moveWarehouse;
    /**
     * 移库入库仓库名称
     */
    private String moveWarehouseName;
    /**
     * 预估运费（元/吨）
     */
    private Float estimatedFreight;


    /**
     * 模糊查询
     */
    @TableField(exist = false)
    private String searchKeyWord;

    /**
     * 累计退库量（吨）
     */
    @TableField(exist = false)
    private Float stockReturnCount;

    /**
     *查询类型
     */
    @TableField(exist = false)
    private String searchType;

    /**
     *移库入库仓库名
     */
    @TableField(exist = false)
    private String warehouseNameIn;

    /**
     * 移库入库重量（吨）
     */
    @TableField(exist = false)
    private Float weightIn;

    @TableField(exist = false)
    private List<ContractManagementInfo> contractManagementInfoList;

    /**
     * 退库详情集合
     */
    @TableField(exist = false)
    private List<InOutWarehouseTask> inOutWarehouseTaskList;
    /**
     * 货源任务关联集合
     */
    @TableField(exist = false)
    private List<SourceRelationInfo> sourceRelationInfoList;
    /**
     * 仓库名集合
     */
    @TableField(exist = false)
    private String stringList;

    @TableField(exist = false)
    private String taskId;
    @TableField(exist = false)
    private String flag;
    /**
     * 退库勾选标识（0未勾选1已勾选2已退库）
     */
    @TableField(exist = false)
    private String returnTickFlag;

    @Override
    protected Serializable pkVal() {
        return this.id;
    }

    public static class QueryFiles {

        public  static final String COMP_ID = "comp_id";

        public static final String MOVE_TASK_NO = "move_task_no";

        public static final String DELETE_FLAG = "delete_flag";

        public static final String relevanceId = "relevance_id";
        public static final String TASK_TYPE_KEY = "task_type_key";

        public static final String SEND_WAREHOUSE = "send_warehouse";

        public static final String RECEIVE_WAREHOUSE = "receive_warehouse";

        public static final String APPROVE_STATUS = "approve_status";



    }

}
