package com.yh.saas.plugin.yiliangyiyun.entity;

import com.baomidou.mybatisplus.annotations.TableField;
import com.baomidou.mybatisplus.annotations.TableId;
import com.baomidou.mybatisplus.annotations.TableName;
import com.baomidou.mybatisplus.enums.IdType;
import java.util.List;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.yh.saas.common.support.entity.BaseModel;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import java.awt.*;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 记录入库单信息
 * </p>
 *
 * @author Gdc
 * @since 2022-05-17
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
@TableName("warehousing_order")
public class WarehousingOrder extends BaseModel<WarehousingOrder> {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @TableId(type = IdType.UUID)
    private String id;
    /**
     * 仓库id
     */
    private String baseId;
    /**
     * 批次id
     */
    private String batchId;
    /**
     * 公司id
     */
    private String compId;
    /**
     * 客户id
     */
    private String identifyId;
    /**
     * 出入库记录id
     */
    private String warehouseInOutId;
    /**
     * 出入库任务id
     */
    private String inOutWarehouseTaskId;
    /**
     * 合同编号
     */
    private String contractNo;
    /**
     * 移库任务编号
     */
    private String moveTaskNo;
    /**
     * 仓库名称
     */
    private String warehouseName;
    /**
     * 入库类型
     */
    private String inType;
    /**
     * 货源
     */
    private String goodsSource;
    /**
     * 车牌号
     */
    private String carNo;
    /**
     * 发货库
     */
    private String deliveryWarehouse;
    /**
     * 货名key
     */
    private String goodsNameKey;
    /**
     * 货名
     */
    private String goodsName;
    /**
     * 入库重量（吨）
     */
    private Float inWarehouseWeight;
    /**
     * 出库重量（吨）
     */
    private Float outWarehouseWeight;
    /**
     * 预计出库重量（吨）
     */
    private Float estimateOutWarehouseWeight;
    /**
     * 本次出库重量（吨）
     */
    private Float outWeight;
    /**
     * 实际出库重量（吨）
     */
    private Float actualOutWarehouseWeight;
    /**
     * 盈亏（吨）
     */
    private Float profitAndLoss;
    /**
     * 审核中的盈亏值（吨）
     */
    private Float plValue;
    /**
     * 操作标识
     */
    private String operationFlag;
    /**
     * 已付(元)
     */
    private Double amountEdPayable;
    /**
     * 未付(元)
     */
    private Double amountNotPayable;
    /**
     * 剩余重量（吨）
     */
    private Float surplusWeight;
    /**
     * 出单时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date issuingTime;
    /**
     * 出库时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date inOutDate;
    /**
     * 粮款(元)
     */
    private Double grainFund;
    /**
     * 费用
     */
    private Double amountMoney;
    /**
     * 请款
     */
    private Double requestFunds;
    /**
     * 请款人
     */
    private String requestPerson;
    /**
     * 请款日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date requestDate;
    /**
     * 客户
     */
    private String customer;
    /**
     * 备注
     */
    private String remark;
    /**
     * 状态
     */
    private String status;
    /**
     * 状态标识
     */
    private String statusFlag;
    /**
     * 盘损状态
     */
    private String inventoryStatus;
    /**
     * 审核状态
     */
    private String approveStatus;
    /**
     * 审核流id
     */
    private String workflowId;
    /**
     * 平均成本（元/吨）
     */
    private Double avgCost;
    /**
     * 附件地址
     */
    private String addressUrl;

    @TableField(exist = false)
    private InOutWarehouseTask inOutWarehouseTask;
    @TableField(exist = false)
    private List<WarehousingOrder> orderList;
    /**
     * 盈亏标识(0盘盈1盘亏)
     */
    @TableField(exist = false)
    private String flag;
    /**
     * 货源标识(0添加1编辑)
     */
    @TableField(exist = false)
    private String taskFlag;

    @TableField(exist = false)
    private String taskId;
    /**
     *查询类型
     */
    @TableField(exist = false)
    private String searchType;
    /**
     * 模糊查询
     */
    @TableField(exist = false)
    private String searchKeyWord;
    /**
     * 开始时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @TableField(exist = false)
    private Date startDate;
    /**
     * 结束时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @TableField(exist = false)
    private Date endDate;
    @TableField(exist = false)
    private List<WarehousingOrder> warehousingOrderList;
    /**
     * 费用分配列表
     */
    @TableField(exist = false)
    private List<ExpenseAllocationInfo> expenseAllocationInfoList;
    /**
     * 请款合计
     */
    @TableField(exist = false)
    private Double requestFundsTotal;
    /**
     * 入库重量合计
     */
    @TableField(exist = false)
    private Float inWarehouseWeightTotal;
    /**
     *車數
     */
    @TableField(exist = false)
    private Integer count;
    /**
     * 仓库类型(1 常用仓库 2 临时仓库)
     */
    @TableField(exist = false)
    private String warehouseType;
    /**
     * 出单时间时分秒
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(exist = false)
    private Date issuingTimeOther;
    /**
     * 出单时间年月日时分秒
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @TableField(exist = false)
    private Date issuingTimeElse;
    /**
     * 本次分配金额
     */
    @TableField(exist = false)
    private Double money;
    /**
     * 出入库任务编号
     */
    @TableField(exist = false)
    private String inOutTaskNo;
    /**
     * 发货库
     */
    @TableField(exist = false)
    private String sendWarehouseName;
    /**
     * 移库出库任务号
     */
    @TableField(exist = false)
    private String sendTaskId;

    @Override
    protected Serializable pkVal() {
        return this.id;
    }

}
